@@ -2,7 +2,7 @@
 #include <Rcpp.h>
 using namespace Rcpp ;
 
-// General to do: Check if you can do it without unsigned int and n[l+1] but just with int and n[l]!
+// General to do: Check if you can do it without unsigned int and n[l+1] but just with int and n[l]
 // also:: perhaps redo everything with data pointers and 2d group indices (instead of filling the 2d structure every time): http://www.cplusplus.com/reference/vector/vector/data/
 // https://stackoverflow.com/questions/1733143/converting-between-c-stdvector-and-c-array-without-copying?rq=1
 // For named vectors, could add right name!
@@ -41,7 +41,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
           if(!index) {
             hash.data[addr] = i+1;
             ++hash.size_;
-            n[i+1] = 1; // good ? stable ?
+            n[i+1] = 1;
             if(nfirstm && max == 1) { // Could also do this at the end in a separate loop. What is faster ? -> This seems better !
               if(minm) {
                 if(mode > val) mode = val;
@@ -68,7 +68,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
             }
           }
         }
-          // if(nfirstm && max == 1) { // Above seems better !!
+          // if(nfirstm && max == 1) { // Above seems better !
           //   if(minm) {
           //     for(int i = 1; i != l; ++i) if(mode > x[i]) mode = x[i];
           //   } else {
@@ -89,7 +89,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
           if(!index) {
             hash.data[addr] = i+1;
             ++hash.size_;
-            n[i+1] = 1; // good ? stable ?
+            n[i+1] = 1;
             if(nfirstm && max == 1) { // Could also do this at the end in a separate loop. What is faster ? -> This seems better !
               if(minm) {
                 if(mode > val) mode = val;
@@ -125,7 +125,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
         // }
       }
       Vector<RTYPE> out(1, mode);
-      DUPLICATE_ATTRIB(out, x); // could add right name for names vectors
+      DUPLICATE_ATTRIB(out, x); // could add right name for named vectors
       if(Rf_getAttrib(x, R_NamesSymbol) != R_NilValue) Rf_setAttrib(out, R_NamesSymbol, R_NilValue);
       return out;
     } else {
@@ -138,7 +138,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
       if(Rf_isNull(gs)) {
         for(int i = 0; i != l; ++i) ++n[pg[i]];
         for(int i = 1; i != ngp; ++i) {
-          if(n[i] == 0) stop("group size of 0 encountered");
+          if(n[i] == 0) stop("Group size of 0 encountered. This is probably due to unused factor levels. Use fdroplevels(f) to drop them.");
           gmap[i] = std::vector<storage_t> (n[i]); // Vector<RTYPE>
           n[i] = 0;
         }
@@ -146,7 +146,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
         IntegerVector gsv = gs;
         if(ng != gsv.size()) stop("ng must match length(gs)");
         for(int i = 0; i != ng; ++i) {
-          if(gsv[i] == 0) stop("group size of 0 encountered");
+          if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably due to unused factor levels. Use fdroplevels(f) to drop them.");
           gmap[i+1] = std::vector<storage_t> (gsv[i]); // Vector<RTYPE>
         }
       }
@@ -156,10 +156,10 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
           // const std::vector<storage_t>& temp = gmap[gr]; // wrap() // good ? // const Vector<RTYPE>& // better for character strings
           sugar::IndexHash<RTYPE> hash(wrap(gmap[gr+1]));  // wrap(temp)
           int i = 0, s = hash.n, end = s-1, max = 1, index; // n[s+1] // fastest ? use n ?
-          std::vector<int> n(s+1); //  = no_init_vector // better for valgrind
           while(isnanT(hash.src[i]) && i!=end) ++i;
           out[gr] = hash.src[i]; // good
           if(i!=end) {
+            std::vector<int> n(s+1); //  = no_init_vector // better for valgrind
             for( ; i != s; ++i) {
             storage_t val = hash.src[i];
             if(isnanT(val)) continue;
@@ -296,7 +296,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
           if(!index) {
             hash.data[addr] = i+1;
             ++hash.size_;
-            n[i+1] = pwg[i]; // good? stable ?
+            n[i+1] = pwg[i];
             if(pwg[i] >= max) { // necessary, because second loop only entered for more than one occurrence of the same value
               if(pwg[i] > max) {
                 max = pwg[i];
@@ -331,6 +331,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
         }
       } else {
         for(int i = 0; i != l; ++i) {
+          if(std::isnan(pwg[i])) continue;
           storage_t val = hash.src[i];
           addr = hash.get_addr(val);
           index = hash.data[addr];
@@ -342,7 +343,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
           if(!index) {
             hash.data[addr] = i+1;
             ++hash.size_;
-            n[i+1] = pwg[i]; // good? stable ?
+            n[i+1] = pwg[i];
             if(pwg[i] >= max) { // necessary, because second loop only entered for more than one occurrence of the same value
               if(pwg[i] > max) {
                 max = pwg[i];
@@ -391,7 +392,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
       if(Rf_isNull(gs)) {
         for(int i = 0; i != l; ++i) ++n[pg[i]];
         for(int i = 1; i != ngp; ++i) {
-          if(n[i] == 0) stop("group size of 0 encountered");
+          if(n[i] == 0) stop("Group size of 0 encountered. This is probably due to unused factor levels. Use fdroplevels(f) to drop them.");
           gmap[i] = std::vector<storage_t> (n[i]);
           wmap[i] = std::vector<double> (n[i]);
           n[i] = 0;
@@ -400,7 +401,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
         IntegerVector gsv = gs;
         if(ng != gsv.size()) stop("ng must match length(gs)");
         for(int i = 0; i != ng; ++i) {
-          if(gsv[i] == 0) stop("group size of 0 encountered");
+          if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably due to unused factor levels. Use fdroplevels(f) to drop them.");
           gmap[i+1] = std::vector<storage_t> (gsv[i]);
           wmap[i+1] = std::vector<double> (gsv[i]);
         }
@@ -417,50 +418,52 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
           sugar::IndexHash<RTYPE> hash(wrap(gmap[gr+1])); // wrap(temp)
           int i = 0, s = hash.n, end = s-1, index;
           double max = DBL_MIN; // n[s+1]
-          std::vector<double> n(s+1); //  = no_init_vector // better for valgrind
           while((isnanT(hash.src[i]) || std::isnan(wtemp[i])) && i!=end) ++i;
           out[gr] = hash.src[i]; // good !
-          if(i!=end) for( ; i != s; ++i) {
-            storage_t val = hash.src[i];
-            if(isnanT(val) || std::isnan(wtemp[i])) continue;
-            addr = hash.get_addr(val);
-            index = hash.data[addr];
-            while(index && hash.not_equal(hash.src[index - 1], val)) {
-              ++addr;
-              if(addr == static_cast<unsigned int>(hash.m)) addr = 0;
+          if(i!=end) {
+            std::vector<double> n(s+1); //  = no_init_vector // better for valgrind
+            for( ; i != s; ++i) {
+              storage_t val = hash.src[i];
+              if(isnanT(val) || std::isnan(wtemp[i])) continue;
+              addr = hash.get_addr(val);
               index = hash.data[addr];
-            }
-            if(!index) {
-              hash.data[addr] = i+1;
-              ++hash.size_;
-              n[i+1] = wtemp[i];
-              if(wtemp[i] >= max) { // necessary, because second loop only entered for more than one occurrence of the same value
-                if(wtemp[i] > max) {
-                  max = wtemp[i];
-                  out[gr] = val;
-                } else if(nfirstm) { // Could also do this at the end in a separate loop. What is faster ??
-                  if(minm) {
-                    if(out[gr] > val) out[gr] = val;
-                  } else {
-                    if(out[gr] < val) out[gr] = val;
+              while(index && hash.not_equal(hash.src[index - 1], val)) {
+                ++addr;
+                if(addr == static_cast<unsigned int>(hash.m)) addr = 0;
+                index = hash.data[addr];
+              }
+              if(!index) {
+                hash.data[addr] = i+1;
+                ++hash.size_;
+                n[i+1] = wtemp[i];
+                if(wtemp[i] >= max) { // necessary, because second loop only entered for more than one occurrence of the same value
+                  if(wtemp[i] > max) {
+                    max = wtemp[i];
+                    out[gr] = val;
+                  } else if(nfirstm) { // Could also do this at the end in a separate loop. What is faster ??
+                    if(minm) {
+                      if(out[gr] > val) out[gr] = val;
+                    } else {
+                      if(out[gr] < val) out[gr] = val;
+                    }
                   }
                 }
-              }
-            } else {
-              n[index] += wtemp[i];
-              // if(n[index] > max) {
-              //   max = n[index];
-              //   out[gr] = val;
-              // }
-              if(n[index] >= max) {
-                if(n[index] > max) {
-                  max = n[index];
-                  out[gr] = val;
-                } else if(nfirstm) {
-                  if(minm) {
-                    if(out[gr] > val) out[gr] = val;
-                  } else {
-                    if(out[gr] < val) out[gr] = val;
+              } else {
+                n[index] += wtemp[i];
+                // if(n[index] > max) {
+                //   max = n[index];
+                //   out[gr] = val;
+                // }
+                if(n[index] >= max) {
+                  if(n[index] > max) {
+                    max = n[index];
+                    out[gr] = val;
+                  } else if(nfirstm) {
+                    if(minm) {
+                      if(out[gr] > val) out[gr] = val;
+                    } else {
+                      if(out[gr] < val) out[gr] = val;
+                    }
                   }
                 }
               }
@@ -477,6 +480,7 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
           double max = DBL_MIN; // n[s+1];
           std::vector<double> n(s+1); //  = no_init_vector // better for valgrind
           for(int i = 0; i != s; ++i) {
+            if(std::isnan(wtemp[i])) continue;
             storage_t val = hash.src[i];
             addr = hash.get_addr(val);
             index = hash.data[addr];
@@ -530,6 +534,289 @@ Vector<RTYPE> fmodeImpl(const Vector<RTYPE>& x, int ng, const IntegerVector& g,
   }
 }
 
+
+IntegerVector fmodeFACT(const IntegerVector& x, int ng, const IntegerVector& g, const SEXP& gs, const SEXP& w, bool narm, int ret) {
+  int l = x.size(), nlevp = Rf_nlevels(x)+1, val = 0;
+  bool nfirstm = ret != 0, minm = ret == 1;
+
+  if(Rf_isNull(w)) { // No Weights
+    if(ng == 0) { // No Groups
+      std::vector<int> n(nlevp);
+      int max = 1, mode = x[0];
+      if(narm) {
+        int i = 0, end = l-1;
+        while(mode == NA_INTEGER && i!=end) mode = x[++i];
+        if(i!=end) {
+          for( ; i != l; ++i) {
+            val = x[i];
+            if(val == NA_INTEGER) continue;
+            if(++n[val] >= max) {
+              if(n[val] > max) {
+                max = n[val];
+                mode = val;
+              } else if(nfirstm) {
+                if(minm) {
+                  if(mode > val) mode = val;
+                } else {
+                  if(mode < val) mode = val;
+                }
+              }
+            }
+          }
+        }
+      } else {
+        for(int i = 0; i != l; ++i) {
+          val = x[i];
+          if(val == NA_INTEGER) val = 0;
+          if(++n[val] >= max) {
+            if(n[val] > max) {
+              max = n[val];
+              mode = val;
+            } else if(nfirstm) {
+              if(minm) {
+                if(mode > val) mode = val;
+              } else {
+                if(mode < val) mode = val;
+              }
+            }
+          }
+        }
+        if(mode == 0) mode = NA_INTEGER;
+      }
+      IntegerVector out(1, mode);
+      DUPLICATE_ATTRIB(out, x); // could add right name for names vectors
+      if(Rf_getAttrib(x, R_NamesSymbol) != R_NilValue) Rf_setAttrib(out, R_NamesSymbol, R_NilValue);
+      return out;
+    } else {
+      if(l != g.size()) stop("length(g) must match length(x)");
+      const int *pg = g.begin();
+      int ngp = ng+1;
+      std::vector<std::vector<int> > gmap(ngp);
+      IntegerVector out = no_init_vector(ng);
+      std::vector<int> n(ngp);
+      if(Rf_isNull(gs)) {
+        for(int i = 0; i != l; ++i) ++n[pg[i]];
+        for(int i = 1; i != ngp; ++i) {
+          if(n[i] == 0) stop("Group size of 0 encountered. This is probably due to unused factor levels. Use fdroplevels(f) to drop them.");
+          gmap[i] = std::vector<int> (n[i]);
+          n[i] = 0;
+        }
+      } else {
+        IntegerVector gsv = gs;
+        if(ng != gsv.size()) stop("ng must match length(gs)");
+        for(int i = 0; i != ng; ++i) {
+          if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably due to unused factor levels. Use fdroplevels(f) to drop them.");
+          gmap[i+1] = std::vector<int> (gsv[i]);
+        }
+      }
+      for(int i = 0; i != l; ++i) gmap[pg[i]][n[pg[i]]++] = x[i];
+      if(narm) {
+        for(int gr = 0; gr != ng; ++gr) {
+          const std::vector<int>& temp = gmap[gr+1];
+          int i = 0, s = temp.size(), end = s-1, max = 1;
+          while(temp[i] == NA_INTEGER && i!=end) ++i;
+          out[gr] = temp[i];
+          if(i!=end) {
+            std::vector<int> n(nlevp);
+            for( ; i != s; ++i) {
+              val = temp[i];
+              if(val == NA_INTEGER) continue;
+              if(++n[val] >= max) {
+                if(n[val] > max) {
+                  max = n[val];
+                  out[gr] = val;
+                } else if(nfirstm) {
+                  if(minm) {
+                    if(out[gr] > val) out[gr] = val;
+                  } else {
+                    if(out[gr] < val) out[gr] = val;
+                  }
+                }
+              }
+            }
+          }
+        }
+      } else {
+        for(int gr = 0; gr != ng; ++gr) {
+          const std::vector<int>& temp = gmap[gr+1];
+          int tl = temp.size(), max = 1;
+          std::vector<int> n(nlevp);
+          out[gr] = temp[0];
+          for(int i = 0; i != tl; ++i) {
+            val = temp[i];
+            if(val == NA_INTEGER) val = 0;
+            if(++n[val] >= max) {
+              if(n[val] > max) {
+                max = n[val];
+                out[gr] = val;
+              } else if(nfirstm) {
+                if(minm) {
+                  if(out[gr] > val) out[gr] = val;
+                } else {
+                  if(out[gr] < val) out[gr] = val;
+                }
+              }
+            }
+          }
+          if(out[gr] == 0) out[gr] = NA_INTEGER;
+        }
+      }
+      DUPLICATE_ATTRIB(out, x);
+      if(Rf_getAttrib(x, R_NamesSymbol) != R_NilValue) Rf_setAttrib(out, R_NamesSymbol, R_NilValue);
+      return out;
+    }
+  } else { // With Weights
+    NumericVector wg = w;
+    if(l != wg.size()) stop("length(w) must match length(x)");
+    double *pwg = wg.begin();
+
+    if(ng == 0) {
+      double max = DBL_MIN;
+      int mode = x[0];
+      std::vector<double> n(nlevp);
+
+      if(narm) {
+        int i = 0, end = l-1;
+        while((mode == NA_INTEGER || std::isnan(pwg[i])) && i!=end) mode = x[++i];
+        if(i!=end) for( ; i != l; ++i) {
+          val = x[i];
+          if(val == NA_INTEGER || std::isnan(pwg[i])) continue;
+          n[val] += pwg[i];
+          if(n[val] >= max) {
+            if(n[val] > max) {
+              max = n[val];
+              mode = val;
+            } else if(nfirstm) {
+              if(minm) {
+                if(mode > val) mode = val;
+              } else {
+                if(mode < val) mode = val;
+              }
+            }
+          }
+        }
+      } else {
+        for(int i = 0; i != l; ++i) {
+          if(std::isnan(pwg[i])) continue;
+          val = x[i];
+          if(val == NA_INTEGER) val = 0;
+          n[val] += pwg[i];
+          if(n[val] >= max) {
+            if(n[val] > max) {
+              max = n[val];
+              mode = val;
+            } else if(nfirstm) {
+              if(minm) {
+                if(mode > val) mode = val;
+              } else {
+                if(mode < val) mode = val;
+              }
+            }
+          }
+        }
+        if(mode == 0) mode = NA_INTEGER;
+      }
+      IntegerVector out(1, mode);
+      DUPLICATE_ATTRIB(out, x);
+      if(Rf_getAttrib(x, R_NamesSymbol) != R_NilValue) Rf_setAttrib(out, R_NamesSymbol, R_NilValue);
+      return out;
+    } else {
+      if(l != g.size()) stop("length(g) must match length(x)");
+      const int *pg = g.begin();
+      int ngp = ng+1;
+      std::vector<std::vector<int> > gmap(ngp);
+      std::vector<std::vector<double> > wmap(ngp);
+      IntegerVector out = no_init_vector(ng);
+      std::vector<int> n(ngp);
+      if(Rf_isNull(gs)) {
+        for(int i = 0; i != l; ++i) ++n[pg[i]];
+        for(int i = 1; i != ngp; ++i) {
+          if(n[i] == 0) stop("Group size of 0 encountered. This is probably due to unused factor levels. Use fdroplevels(f) to drop them.");
+          gmap[i] = std::vector<int> (n[i]);
+          wmap[i] = std::vector<double> (n[i]);
+          n[i] = 0;
+        }
+      } else {
+        IntegerVector gsv = gs;
+        if(ng != gsv.size()) stop("ng must match length(gs)");
+        for(int i = 0; i != ng; ++i) {
+          if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably due to unused factor levels. Use fdroplevels(f) to drop them.");
+          gmap[i+1] = std::vector<int> (gsv[i]);
+          wmap[i+1] = std::vector<double> (gsv[i]);
+        }
+      }
+      for(int i = 0; i != l; ++i) {
+        int gi = pg[i];
+        gmap[gi][n[gi]] = x[i];
+        wmap[gi][n[gi]++] = pwg[i];
+      }
+      if(narm) {
+        for(int gr = 0; gr != ng; ++gr) {
+          const std::vector<int>& temp = gmap[gr+1];
+          const std::vector<double>& wtemp = wmap[gr+1];
+          int i = 0, s = temp.size(), end = s-1;
+          double max = DBL_MIN;
+          while((temp[i] == NA_INTEGER || std::isnan(wtemp[i])) && i!=end) ++i;
+          out[gr] = temp[i];
+          if(i!=end) {
+            std::vector<double> n(nlevp);
+            for( ; i != s; ++i) {
+              val = temp[i];
+              if(val == NA_INTEGER || std::isnan(wtemp[i])) continue;
+              n[val] += wtemp[i];
+              if(n[val] >= max) {
+                if(n[val] > max) {
+                  max = n[val];
+                  out[gr] = val;
+                } else if(nfirstm) {
+                  if(minm) {
+                    if(out[gr] > val) out[gr] = val;
+                  } else {
+                    if(out[gr] < val) out[gr] = val;
+                  }
+                }
+              }
+            }
+          }
+        }
+      } else {
+        for(int gr = 0; gr != ng; ++gr) {
+          const std::vector<int>& temp = gmap[gr+1];
+          const std::vector<double>& wtemp = wmap[gr+1];
+          int tl = temp.size();
+          double max = DBL_MIN;
+          std::vector<double> n(nlevp);
+          out[gr] = temp[0];
+          for(int i = 0; i != tl; ++i) {
+            if(std::isnan(wtemp[i])) continue;
+            val = temp[i];
+            if(val == NA_INTEGER) val = 0;
+            n[val] += wtemp[i];
+            if(n[val] >= max) {
+              if(n[val] > max) {
+                max = n[val];
+                out[gr] = val;
+              } else if(nfirstm) {
+                if(minm) {
+                  if(out[gr] > val) out[gr] = val;
+                } else {
+                  if(out[gr] < val) out[gr] = val;
+                }
+              }
+            }
+          }
+          if(out[gr] == 0) out[gr] = NA_INTEGER;
+        }
+      }
+      DUPLICATE_ATTRIB(out, x);
+      if(Rf_getAttrib(x, R_NamesSymbol) != R_NilValue) Rf_setAttrib(out, R_NamesSymbol, R_NilValue);
+      return out;
+    }
+  }
+}
+
+
+
 template <> // No logical vector with sugar::IndexHash<RTYPE> !
 Vector<LGLSXP> fmodeImpl(const Vector<LGLSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, const SEXP& w, bool narm, int ret) {
   int l = x.size();
@@ -675,29 +962,19 @@ Vector<LGLSXP> fmodeImpl(const Vector<LGLSXP>& x, int ng, const IntegerVector& g
   }
 }
 
-template <>
-Vector<CPLXSXP> fmodeImpl(const Vector<CPLXSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, const SEXP& w, bool narm, int ret) {
-  stop("Not supported SEXP type!");
-}
-
-template <>
-Vector<VECSXP> fmodeImpl(const Vector<VECSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, const SEXP& w, bool narm, int ret) {
-  stop("Not supported SEXP type!");
-}
-
-template <>
-Vector<RAWSXP> fmodeImpl(const Vector<RAWSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, const SEXP& w, bool narm, int ret) {
-  stop("Not supported SEXP type!");
-}
-
-template <>
-Vector<EXPRSXP> fmodeImpl(const Vector<EXPRSXP>& x, int ng, const IntegerVector& g, const SEXP& gs, const SEXP& w, bool narm, int ret) {
-  stop("Not supported SEXP type!");
-}
 
 // [[Rcpp::export]]
-SEXP fmodeCpp(SEXP x, int ng = 0, IntegerVector g = 0, SEXP gs = R_NilValue, SEXP w = R_NilValue, bool narm = true, int ret = 0) {
-  RCPP_RETURN_VECTOR(fmodeImpl, x, ng, g, gs, w, narm, ret);
+SEXP fmodeCpp(const SEXP& x, int ng = 0, const IntegerVector& g = 0, const SEXP& gs = R_NilValue, const SEXP& w = R_NilValue, bool narm = true, int ret = 0) {
+  switch(TYPEOF(x)) {
+  case REALSXP: return fmodeImpl<REALSXP>(x, ng, g, gs, w, narm, ret);
+  case INTSXP:
+    if(Rf_isFactor(x) && (ng == 0 || Rf_nlevels(x) < Rf_length(x) / ng * 3))
+      return fmodeFACT(x, ng, g, gs, w, narm, ret);
+    return fmodeImpl<INTSXP>(x, ng, g, gs, w, narm, ret);
+  case STRSXP: return fmodeImpl<STRSXP>(x, ng, g, gs, w, narm, ret);
+  case LGLSXP: return fmodeImpl<LGLSXP>(x, ng, g, gs, w, narm, ret);
+  default: stop("Not supported SEXP type !");
+  }
 }
 
 
@@ -713,7 +990,9 @@ SEXP fmodelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEXP
       out[j] = fmodeImpl<REALSXP>(x[j], ng, g, gs, w, narm, ret);
       break;
     case INTSXP:
-      out[j] = fmodeImpl<INTSXP>(x[j], ng, g, gs, w, narm, ret);
+      if(Rf_isFactor(x[j]) && (ng == 0 || Rf_nlevels(x[j]) < Rf_length(x[j]) / ng * 3))
+        out[j] = fmodeFACT(x[j], ng, g, gs, w, narm, ret);
+      else out[j] = fmodeImpl<INTSXP>(x[j], ng, g, gs, w, narm, ret);
       break;
     case STRSXP:
       out[j] = fmodeImpl<STRSXP>(x[j], ng, g, gs, w, narm, ret);
@@ -725,8 +1004,8 @@ SEXP fmodelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEXP
     }
   }
   DUPLICATE_ATTRIB(out, x);
-  if(ng == 0) out.attr("row.names") = 1;
-  else out.attr("row.names") = IntegerVector::create(NA_INTEGER, -ng);
+  if(ng == 0) Rf_setAttrib(out, R_RowNamesSymbol, Rf_ScalarInteger(1));
+  else Rf_setAttrib(out, R_RowNamesSymbol, IntegerVector::create(NA_INTEGER, -ng));
   return out;
 }
 
@@ -738,11 +1017,12 @@ SEXP fmodemImpl(const Matrix<RTYPE>& x, int ng, const IntegerVector& g,
   int col = x.ncol();
   Matrix<RTYPE> out = (ng == 0) ? no_init_matrix(1, col) : no_init_matrix(ng, col);
   for(int j = col; j--; ) out(_, j) = fmodeImpl<RTYPE>(x(_, j), ng, g, gs, w, narm, ret);
-  if(drop) {
-    Rf_setAttrib(out, R_DimSymbol, R_NilValue);
+  if(drop && ng == 0) {
+    Rf_setAttrib(out, R_DimSymbol, R_NilValue); // Rf_dimgets(out, R_NilValue); -> Doesn't work !
     Rf_setAttrib(out, R_NamesSymbol, colnames(x));
   } else {
     colnames(out) = colnames(x);
+    if(!Rf_isObject(x)) Rf_copyMostAttrib(x, out);
   }
   return out;
 }
