@@ -1,16 +1,17 @@
 #include <Rcpp.h>
 using namespace Rcpp;
 
+// Notes:
 // for mean there are 2 options: "overall.mean" = R_NegInf adds the overall mean. default is centering on 0, or centering on a mean provided, or FALSE = R_PosInf -> no centering, scaling preserves mean
 // for sd there is "within.sd" = R_NegInf, scaling by the frequency weighted within-group sd, default is 1, or scaling by a sd provided.
-// Note: All other comments are in fvar.cpp
+// All other comments are in fvar.cpp (in C++ folder, not on Github)
 
 // [[Rcpp::export]]
 NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector& g = 0, const SEXP& w = R_NilValue,
                         bool narm = true, double set_mean = 0, double set_sd = 1) { // could set mean and sd with SEXP, but complicated...
   int l = x.size();
   NumericVector out = no_init_vector(l);
-  //   DUPLICATE_ATTRIB(out, x); // Any speed loss or overwriting attributes ??
+  //   DUPLICATE_ATTRIB(out, x); // Any speed loss or overwriting attributes ?
   if (Rf_isNull(w)) { // No weights
     if (ng == 0) {
       if(set_sd == R_NegInf) stop("within.sd can only be calculated when a grouping vector is supplied");
@@ -26,7 +27,7 @@ NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector&
             mean += d1 * (1 / ++n);
             M2 += d1*(x[i]-mean);
           }
-          M2 = set_sd/sqrt(M2/(n-1)); // good ?? -> Yes, works !!
+          M2 = set_sd/sqrt(M2/(n-1)); // good ? -> Yes, works !
         } else { // use goto to make code simpler ?
           std::fill(out.begin(), out.end(), NA_REAL);
           DUPLICATE_ATTRIB(out, x);
@@ -50,13 +51,13 @@ NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector&
         std::fill(out.begin(), out.end(), NA_REAL);
       } else {
         if(set_mean == 0) out = (x-mean)*M2;
-        else if(set_mean == R_PosInf) out = (x-mean)*M2 + mean; // best ?? // !R_FINITE(set_mean)
-        else out = (x-mean)*M2 + set_mean; // best ??
+        else if(set_mean == R_PosInf) out = (x-mean)*M2 + mean; // best ? // !R_FINITE(set_mean)
+        else out = (x-mean)*M2 + set_mean; // best ?
       }
     } else { // with groups
       if(g.size() != l) stop("length(g) must match nrow(X)");
-      double d1 = 0, gl_mean = 0; // Best way of doing this ?? How can you declare variables in global scope ??
-      // NumericVector mean =  narm ? no_init_vector(ng) : NumericVector(ng); // works but valgrind issue !!
+      double d1 = 0, gl_mean = 0; // Best way of doing this ? How can you declare variables in global scope ?
+      // NumericVector mean =  narm ? no_init_vector(ng) : NumericVector(ng); // works but valgrind issue
       // NumericVector M2 = narm ? NumericVector(ng, NA_REAL) : NumericVector(ng);
       // NumericVector n = narm ? NumericVector(ng, 1.0) : NumericVector(ng);
       NumericVector mean(ng), n(ng, (narm) ? 1.0 : 0.0), M2(ng, (narm) ? NA_REAL : 0.0);
@@ -113,7 +114,7 @@ NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector&
           gl_mean = set_mean;
         }
         within_sd = sqrt(within_sd/(sum_n-1));
-        M2 = M2 * within_sd; // fastest ??
+        M2 = M2 * within_sd; // fastest ?
       } else {
         if(set_mean == R_NegInf) {
           int sum_n = 0;
@@ -132,9 +133,9 @@ NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector&
       if(set_mean == 0) {
         for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1];
       } else if(set_mean == R_PosInf) {
-        for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1] + mean[g[i]-1]; // best ??
+        for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1] + mean[g[i]-1]; // best ?
       } else {
-        for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1] + gl_mean; // best ??
+        for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1] + gl_mean; // best ?
       }
     }
   } else { // With weights
@@ -179,14 +180,14 @@ NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector&
         std::fill(out.begin(), out.end(), NA_REAL);
       } else {
         if(set_mean == 0) out = (x-mean)*M2;
-        else if(set_mean == R_PosInf) out = (x-mean)*M2 + mean; // best ??
-        else out = (x-mean)*M2 + set_mean; // best ??
+        else if(set_mean == R_PosInf) out = (x-mean)*M2 + mean; // best ?
+        else out = (x-mean)*M2 + set_mean; // best ?
       }
     } else { // with groups
       if(g.size() != l) stop("length(g) must match nrow(X)");
-      double d1 = 0, gl_mean = 0; // Best way of doing this ?? How can you declare variables in overall scope ??
+      double d1 = 0, gl_mean = 0; // Best way of doing this ? How can you declare variables in overall scope ?
       // NumericVector M2 = narm ? NumericVector(ng, NA_REAL) : NumericVector(ng);
-      NumericVector M2(ng, (narm) ? NA_REAL : 0.0), mean(ng), sumw(ng); // = narm ? no_init_vector(ng) : NumericVector(ng); // works but valgrind issues !!!
+      NumericVector M2(ng, (narm) ? NA_REAL : 0.0), mean(ng), sumw(ng); // = narm ? no_init_vector(ng) : NumericVector(ng); // works but valgrind issues
       // NumericVector sumw = narm ? no_init_vector(ng) : NumericVector(ng);
       if(narm) {
         for(int i = l; i--; ) {
@@ -243,7 +244,7 @@ NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector&
           gl_mean = set_mean;
         }
         within_sd = sqrt(within_sd/(sum_sumw-1));
-        M2 = M2 * within_sd; // fastest ??
+        M2 = M2 * within_sd; // fastest ?
       } else {
         if(set_mean == R_NegInf) {
           double sum_sumw = 0;
@@ -262,9 +263,9 @@ NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector&
       if(set_mean == 0) {
         for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1];
       } else if(set_mean == R_PosInf) {
-        for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1] + mean[g[i]-1]; // best ??
+        for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1] + mean[g[i]-1]; // best ?
       } else {
-        for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1] + gl_mean; // best ??
+        for(int i = 0; i != l; ++i) out[i] = (x[i]-mean[g[i]-1])*M2[g[i]-1] + gl_mean; // best ?
       }
     }
   }
@@ -273,7 +274,6 @@ NumericVector fscaleCpp(const NumericVector& x, int ng = 0, const IntegerVector&
 }
 
 
-// Still thoroughly check this one and the data.frame version !!
 // [[Rcpp::export]]
 NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector& g = 0, const SEXP& w = R_NilValue,
                          bool narm = true, double set_mean = 0, double set_sd = 1) {
@@ -289,7 +289,7 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
         NumericMatrix::ConstColumn column = x( _ , j);
         NumericMatrix::Column outj = out( _ , j);
         double nj = 0, meanj = 0, d1 = 0, M2j = 0;
-        if(narm) { // faster using 2 loops over columns ???
+        if(narm) { // faster using 2 loops over columns ?
           int k = l-1;
           while(std::isnan(column[k]) && k!=0) --k;
           if(k != 0) {
@@ -302,7 +302,7 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
             M2j = set_sd/sqrt(M2j/(nj-1));
           } else {
             std::fill(outj.begin(), outj.end(), NA_REAL);
-            continue; // Necessary !!
+            continue; // Necessary
           }
         } else {
           for(int i = 0; i != l; ++i) {
@@ -321,15 +321,15 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
           std::fill(outj.begin(), outj.end(), NA_REAL);
         } else {
           if(set_mean == 0) outj = (column-meanj)*M2j;
-          else if(set_mean == R_PosInf) outj = (column-meanj)*M2j + meanj; // best ??
-          else outj = (column-meanj)*M2j + set_mean; // best ??
+          else if(set_mean == R_PosInf) outj = (column-meanj)*M2j + meanj; // best ?
+          else outj = (column-meanj)*M2j + set_mean; // best ?
         }
       }
     } else { // with groups
       if(g.size() != l) stop("length(g) must match nrow(X)");
-      // Better way ??
+      // Better way ?
       NumericVector meanj(ng), nj(ng), M2j(ng);
-      // NumericVector meanj = no_init_vector(ng), nj = no_init_vector(ng), M2j = no_init_vector(ng); // Works but valgrind issue !!
+      // NumericVector meanj = no_init_vector(ng), nj = no_init_vector(ng), M2j = no_init_vector(ng); // Works but valgrind issue
       for(int j = col; j--; ) {
         NumericMatrix::ConstColumn column = x( _ , j);
         NumericMatrix::Column outj = out( _ , j);
@@ -367,7 +367,7 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
             }
           }
         }
-        if(set_sd == R_NegInf) { // best way of coding ?? Goes through all the if conditions for every column...
+        if(set_sd == R_NegInf) { // best way of coding ? Goes through all the if conditions for every column...
           double within_sdj = 0;
           int sum_nj = 0;
           if(set_mean == R_NegInf) {
@@ -389,7 +389,7 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
             gl_meanj = set_mean;
           }
           within_sdj = sqrt(within_sdj/(sum_nj-1));
-          M2j = M2j * within_sdj; // fastest ??
+          M2j = M2j * within_sdj; // fastest ?
         } else {
           if(set_mean == R_NegInf) {
             int sum_nj = 0;
@@ -408,9 +408,9 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
         if(set_mean == 0) {
           for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1];
         } else if(set_mean == R_PosInf) {
-          for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + meanj[g[i]-1]; // best ??
+          for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + meanj[g[i]-1]; // best ?
         } else {
-          for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + gl_meanj; // best ??
+          for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + gl_meanj; // best ?
         }
         loopend:;
       }
@@ -438,7 +438,7 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
             }
           } else {
             std::fill(outj.begin(), outj.end(), NA_REAL);
-            continue; // Necessary !!
+            continue; // Necessary
           }
         } else {
           for(int i = 0; i != l; ++i) {
@@ -458,15 +458,15 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
           std::fill(outj.begin(), outj.end(), NA_REAL);
         } else {
           if(set_mean == 0) outj = (column-meanj)*M2j;
-          else if(set_mean == R_PosInf) outj = (column-meanj)*M2j + meanj; // best ??
-          else outj = (column-meanj)*M2j + set_mean; // best ??
+          else if(set_mean == R_PosInf) outj = (column-meanj)*M2j + meanj; // best ?
+          else outj = (column-meanj)*M2j + set_mean; // best ?
         }
       }
     } else { // with groups and weights
       if(g.size() != l) stop("length(g) must match nrow(X)");
-      // Works but valgrind issue !!
-      // NumericVector meanj = no_init_vector(ng), sumwj = no_init_vector(ng), M2j = no_init_vector(ng); // stable ??
-      NumericVector meanj(ng), sumwj(ng), M2j(ng); // better for valgrind !!
+      // Works but valgrind issue
+      // NumericVector meanj = no_init_vector(ng), sumwj = no_init_vector(ng), M2j = no_init_vector(ng);
+      NumericVector meanj(ng), sumwj(ng), M2j(ng); // better for valgrind
       for(int j = col; j--; ) {
         NumericMatrix::ConstColumn column = x( _ , j);
         NumericMatrix::Column outj = out( _ , j);
@@ -506,7 +506,7 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
             }
           }
         }
-        if(set_sd == R_NegInf) { // best way of coding ?? Goes through all the if conditions for every column...
+        if(set_sd == R_NegInf) { // best way of coding ? Goes through all the if conditions for every column...
           double within_sdj = 0, sum_sumwj = 0;
           if(set_mean == R_NegInf) {
             for(int i = ng; i--; ) {
@@ -527,7 +527,7 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
             gl_meanj = set_mean;
           }
           within_sdj = sqrt(within_sdj/(sum_sumwj-1));
-          M2j = M2j * within_sdj; // fastest ??
+          M2j = M2j * within_sdj; // fastest ?
         } else {
           if(set_mean == R_NegInf) {
             double sum_sumwj = 0;
@@ -546,9 +546,9 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
         if(set_mean == 0) {
           for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1];
         } else if(set_mean == R_PosInf) {
-          for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + meanj[g[i]-1]; // best ??
+          for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + meanj[g[i]-1]; // best ?
         } else {
-          for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + gl_meanj; // best ??
+          for(int i = 0; i != l; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + gl_meanj; // best ?
         }
         loopend2:;
       }
@@ -558,7 +558,6 @@ NumericMatrix fscalemCpp(const NumericMatrix& x, int ng = 0, const IntegerVector
   return out;
 }
 
-// Still thoroughly check !!
 // [[Rcpp::export]]
 List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEXP& w = R_NilValue,
                 bool narm = true, double set_mean = 0, double set_sd = 1) {
@@ -573,7 +572,7 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
       for(int j = l; j--; ) {
         NumericVector column = x[j];
         int row = column.size();
-        NumericVector outj = no_init_vector(row); // outj = NULL gives compile warning
+        NumericVector outj = no_init_vector(row);
         double nj = 0, meanj = 0, d1 = 0, M2j = 0;
         if(narm) {
           int k = row-1;
@@ -587,8 +586,8 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
             }
             M2j = set_sd/sqrt(M2j/(nj-1));
           } else {
-            std::fill(outj.begin(), outj.end(), NA_REAL); // outj = rep(NA_REAL, row); // fastest option !! (really faster than std::fill ??
-            goto loopend; // Necessary !!
+            std::fill(outj.begin(), outj.end(), NA_REAL); // outj = rep(NA_REAL, row); // fastest option ! (faster than std::fill)
+            goto loopend; // Necessary
           }
         } else {
           for(int i = 0; i != row; ++i) {
@@ -607,8 +606,8 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
           std::fill(outj.begin(), outj.end(), NA_REAL);
         } else {
           if(set_mean == 0) outj = (column-meanj)*M2j;
-          else if(set_mean == R_PosInf) outj = (column-meanj)*M2j + meanj; // best ??
-          else outj = (column-meanj)*M2j + set_mean; // best ??
+          else if(set_mean == R_PosInf) outj = (column-meanj)*M2j + meanj; // best ?
+          else outj = (column-meanj)*M2j + set_mean; // best ?
         }
         loopend:;
         SHALLOW_DUPLICATE_ATTRIB(outj, column);
@@ -616,15 +615,15 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
       }
     } else { // with groups
       int gss = g.size();
-      // Better way ??
+      // Better way ?
       NumericVector meanj(ng), nj(ng), M2j(ng);
-      // NumericVector meanj = no_init_vector(ng), nj = no_init_vector(ng), M2j = no_init_vector(ng); // Works but valgrind issue !!
+      // NumericVector meanj = no_init_vector(ng), nj = no_init_vector(ng), M2j = no_init_vector(ng); // Works but valgrind issue
       for(int j = l; j--; ) {
         NumericVector column = x[j];
         if(gss != column.size()) stop("length(g) must match nrow(X)");
         NumericVector outj = no_init_vector(gss);
         double d1 = 0, gl_meanj = 0;
-        if(narm) { // better do two loops ??
+        if(narm) { // better do two loops ?
           std::fill(M2j.begin(), M2j.end(), NA_REAL);
           for(int i = gss; i--; ) {
             if(std::isnan(column[i])) continue;
@@ -657,7 +656,7 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
             }
           }
         }
-        if(set_sd == R_NegInf) { // best way of coding ?? Goes through all the if conditions for every column...
+        if(set_sd == R_NegInf) { // best way of coding ? Goes through all the if conditions for every column...
           double within_sdj = 0;
           int sum_nj = 0;
           if(set_mean == R_NegInf) {
@@ -679,7 +678,7 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
             gl_meanj = set_mean;
           }
           within_sdj = sqrt(within_sdj/(sum_nj-1));
-          M2j = M2j * within_sdj; // fastest ??
+          M2j = M2j * within_sdj; // fastest ?
         } else {
           if(set_mean == R_NegInf) {
             int sum_nj = 0;
@@ -698,9 +697,9 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
         if(set_mean == 0) {
           for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1];
         } else if(set_mean == R_PosInf) {
-          for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + meanj[g[i]-1]; // best ??
+          for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + meanj[g[i]-1]; // best ?
         } else {
-          for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + gl_meanj; // best ??
+          for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + gl_meanj; // best ?
         }
         loopend2:;
         SHALLOW_DUPLICATE_ATTRIB(outj, column);
@@ -731,7 +730,7 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
             }
           } else {
             std::fill(outj.begin(), outj.end(), NA_REAL);
-            goto loopend3; // Necessary !!
+            goto loopend3; // Necessary
           }
         } else {
           for(int i = 0; i != wgs; ++i) {
@@ -751,8 +750,8 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
           std::fill(outj.begin(), outj.end(), NA_REAL);
         } else {
           if(set_mean == 0) outj = (column-meanj)*M2j;
-          else if(set_mean == R_PosInf) outj = (column-meanj)*M2j + meanj; // best ??
-          else outj = (column-meanj)*M2j + set_mean; // best ??
+          else if(set_mean == R_PosInf) outj = (column-meanj)*M2j + meanj; // best ?
+          else outj = (column-meanj)*M2j + set_mean; // best ?
         }
         loopend3:;
         SHALLOW_DUPLICATE_ATTRIB(outj, column);
@@ -762,7 +761,7 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
       int gss = g.size();
       if(gss != wgs) stop("length(w) must match length(g)");
       NumericVector meanj(ng), sumwj(ng), M2j(ng);
-      // NumericVector meanj = no_init_vector(ng), sumwj = no_init_vector(ng), M2j = no_init_vector(ng); // Works but valgrind issue !!
+      // NumericVector meanj = no_init_vector(ng), sumwj = no_init_vector(ng), M2j = no_init_vector(ng); // Works but valgrind issue
       for(int j = l; j--; ) {
         NumericVector column = x[j];
         if(gss != column.size()) stop("length(g) must match nrow(X)");
@@ -803,7 +802,7 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
             }
           }
         }
-        if(set_sd == R_NegInf) { // best way of coding ?? Goes through all the if conditions for every column...
+        if(set_sd == R_NegInf) { // best way of coding ? Goes through all the if conditions for every column...
           double within_sdj = 0, sum_sumwj = 0;
           if(set_mean == R_NegInf) {
             for(int i = ng; i--; ) {
@@ -824,7 +823,7 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
             gl_meanj = set_mean;
           }
           within_sdj = sqrt(within_sdj/(sum_sumwj-1));
-          M2j = M2j * within_sdj; // fastest ??
+          M2j = M2j * within_sdj; // fastest ?
         } else {
           if(set_mean == R_NegInf) {
             double sum_sumwj = 0;
@@ -843,9 +842,9 @@ List fscalelCpp(const List& x, int ng = 0, const IntegerVector& g = 0, const SEX
         if(set_mean == 0) {
           for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1];
         } else if(set_mean == R_PosInf) {
-          for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + meanj[g[i]-1]; // best ??
+          for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + meanj[g[i]-1]; // best ?
         } else {
-          for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + gl_meanj; // best ??
+          for(int i = 0; i != gss; ++i) outj[i] = (column[i]-meanj[g[i]-1])*M2j[g[i]-1] + gl_meanj; // best ?
         }
         loopend4:;
         SHALLOW_DUPLICATE_ATTRIB(outj, column);
